/*
  This file is part of KMail, the KDE mail client.
  Copyright (c) 2009 Sebastian Trueg <trueg@kde.org>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#include "nepomukwidget.h"

#include "kmreaderwin.h"
#include "kmmessage.h"

#include <nepomuk/annotationwidget.h>
#include <nepomuk/kmetadatatagwidget.h>
#include <nepomuk/kratingwidget.h>

#include <QHBoxLayout>
#include <QLabel>
#include <KIcon>
#include <KMessageBox>
#include <QPushButton>
#include <KLocale>

#include <Nepomuk/Resource>
#include <Nepomuk/Variant>
#include <Nepomuk/Thing>

#include <tasktreedialog.h>
#include <KDebug>

static const char* pimoIsRelated = "http://www.semanticdesktop.org/ontologies/2007/11/01/pimo#isRelated";

NepomukWidget::NepomukWidget( QWidget* parent )
  : QWidget( parent )
{
  QHBoxLayout* lay = new QHBoxLayout(this);
  m_tagWidget = new Nepomuk::TagWidget(this);
  m_ratingWidget = new KRatingWidget(this);
  m_annotationWidget = new Nepomuk::AnnotationWidget(this);
  m_tasksButton = new QPushButton( KIcon("task-new"), i18n("Associate to..."), this );
  m_hideButton = new QPushButton( KIcon("arrow-right-double"), "", this);
  m_hideButton->setToolTip( i18n("Hide/show the annotation suggestions") );
  m_hideButton->setCheckable( true );

  lay->addWidget(new QLabel(i18n("Tags:"), this));
  lay->addWidget(m_tagWidget);
  lay->addWidget(m_ratingWidget);
  lay->addWidget(m_tasksButton);
  lay->addWidget(m_annotationWidget);
  lay->addWidget(m_hideButton);

  connect( m_ratingWidget, SIGNAL( ratingChanged( int ) ),
           this, SLOT( slotRatingChanged( int ) ) );
  connect( m_annotationWidget, SIGNAL(annotationAdded(Nepomuk::Resource)),
           this, SIGNAL(changed()));

  connect( m_tasksButton, SIGNAL( clicked( bool )),
           this, SLOT( associateToATask() ));
			   
  connect( m_hideButton, SIGNAL( toggled( bool )),
	   this, SLOT( hideAnnotationPanel( bool )) );
}

NepomukWidget::~NepomukWidget()
{
}

void NepomukWidget::hideAnnotationPanel( bool toggled )
{
  m_hideButton->setIcon( KIcon( toggled ? "arrow-left-double" : "arrow-right-double") );
}

void NepomukWidget::setKMReaderWin( KMReaderWin* kmreaderwin)
{
  m_kmreaderwin = kmreaderwin;
}

void NepomukWidget::associateToATask()
{
  Nepomuk::Resource task( TaskTreeDialog::selectOneTask(NULL) );
  if ( task.isValid() ) {
    Nepomuk::Resource mail = m_kmreaderwin->message()->nepomukResource();
    
    mail.addProperty( QUrl::fromEncoded( pimoIsRelated ), task );
    mail.pimoThing().addProperty( QUrl::fromEncoded( pimoIsRelated ), task ); //evil hack for trug pimo:thing

    m_tasksButton->setText( task.genericLabel() );
    emit changed();

    Nepomuk::Resource toto("TOTOTATATITI", QUrl::fromEncoded("http://www.semanticdesktop.org/ontologies/2007/11/01/pimo#person"));
    toto.setLabel("toto");
    toto.pimoThing().setLabel("toto thing");

  }
}

void NepomukWidget::setResource( const Nepomuk::Resource& res )
{
  m_resource = res;
  m_tagWidget->setTaggedResource( res );
  m_annotationWidget->setResource( res );
  m_ratingWidget->setRating( res.rating() );

  Nepomuk::Variant tasks = res.property( QUrl::fromEncoded(pimoIsRelated) );
  // QHash<QUrl, Nepomuk::Variant> p = res.properties();
  // QHashIterator<QUrl, Nepomuk::Variant>  it(p);
  // QString txt;
  // while ( it.hasNext() ) {
  //   it.next();
  //   txt += it.key().toString() + " " + it.value().toString() + "\n";
  // }
  //KMessageBox::information(this, txt);
  // kDebug() << txt;

  if ( !tasks.toResourceList().isEmpty() ) {
    m_tasksButton->setText( tasks.toResourceList().front().label() );
  }
  else {
    m_tasksButton->setText( i18n("Associate to...") );
  }

}

void NepomukWidget::slotRatingChanged(int rating)
{
  m_resource.setRating(rating);
  emit changed();
}

#include "nepomukwidget.moc"
